local tablet = AceLibrary("Tablet-2.0")
local deformat = AceLibrary("Deformat-2.0")
local PT = AceLibrary("PeriodicTable-3.0")

AlchemyFu = AceLibrary("AceAddon-2.0"):new(
"FuBarPlugin-2.0", 
"AceEvent-2.0", 
"AceConsole-2.0", 
"AceDB-2.0")

local addon = AlchemyFu

local options = {
	type = "group",
	args = {
		reset = {
			type = "execute",
			name = "Reset statistics",
			desc = "Reset all data collected.",
			func = function(v)
				AlchemyFu:ResetDB()
				AlchemyFu:UpdateTooltip()
			end,
		},
	},
}

addon.hasIcon = "Interface\\Icons\\Trade_Alchemy"

function addon:OnInitialize()
	local revision = tonumber((string.match("$Revision: 31661 $", "^.-(%d+).-$"))) or 1
	if not self.version then self.version = "1.0" end
	self.version = self.version .. "." .. revision
	self.revision = revision
	
	self.name = "AlchemyFu"
	self:RegisterDB("AlchemyFuDB")
	self:RegisterDefaults("profile", {
		counts = {},
		creations = {},
	})
	self.OnMenuRequest = options
end

function addon:OnEnable()
	self:RegisterEvent("CHAT_MSG_LOOT")
end

function addon:CHAT_MSG_LOOT(msg)
	local item, amount = deformat(msg, LOOT_ITEM_CREATED_SELF_MULTIPLE)
	if not item or not amount then
		item, amount = deformat(msg, LOOT_ITEM_CREATED_SELF), 1
	end
	
	if not item then return end

	local itemName, itemLink, itemRarity = GetItemInfo(item)
	if not itemLink then return	end
	
	local itemId = tonumber(select(3, itemLink:find("item:(%d+):")))
	if not itemId or not PT:ItemInSet(itemId, "Tradeskill.Crafted.Alchemy") then return	end

	-- Track creations, how many we have made of one item
	self.db.profile.creations[itemId] = self.db.profile.creations[itemId] or 0 + amount

	-- Track procs
	if not self.db.profile.counts[itemId] then
		self.db.profile.counts[itemId] = {}
	end
	
	if not self.db.profile.counts[itemId][amount] then
		self.db.profile.counts[itemId][amount] = 1
	else
		self.db.profile.counts[itemId][amount] = self.db.profile.counts[itemId][amount] + 1
	end
	
	self:UpdateTooltip()
end

function addon:OnTooltipUpdate()
	local cat = tablet:AddCategory(
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)
	
	local total, totalProc = 0, 0
	for itemId, amount in pairs(self.db.profile.creations) do
		cat:AddLine(
			'text', select(2, GetItemInfo(itemId)),
			'text2', amount
		)
		
		for amount, count in pairs(self.db.profile.counts[itemId]) do
			cat:AddLine(
				'text', "   x".. amount,
				'text2', count
			)
			if amount > 1 then
				totalProc = totalProc + (amount * count)
			end
		end
		
		total = total + amount
	end
	
	cat:AddLine(
		'text', "Totally created",
		'text2', total+totalProc
	)
	
	cat:AddLine(
		'text', "Totally created w/o procs",
		'text2', total
	)
	cat:AddLine(
		'text', "Totally created /w procs",
		'text2', totalProc
	)
end